/***************************************************************************************************
   卧室（小台灯）: 1.1.6版+7修订版
   客厅: 1.1.6版+7修订版
   阳台: 1.1.6版+7修订版
  /***************************************************************************************************
  功能介绍：
    1、控制：
      通过发送on\off控制开关、使用米家、小度、天猫精灵等控制开关、GPIO2点触GND引脚控制开关
    2、配网：
      关注微信公众号巴法云，在底部 配网--微信配网或者通过airKiss，即可配置网络（配网时关闭5G信号，或者取消名称合并）
    3、重置配网：
      5秒内连续开关3次，进行重置配网
    4、断电记忆模式切换：
      通过发送keep、inon、inoff（1：保持, 2：常开, 3：常关）进行切换
    5、通过MQTT设备云方式接入网络
  版本号：1.1.6
    1、机械开关方式有问题，短接电流比较大
      将：
      //pinMode(LED_BUILTIN, OUTPUT);
      改为：
      pinMode(LED_BUILTIN, INPUT_PULLUP);
      机械开关进行点动控制
      问题解决
    2、显示设备不在线：
        MQTT前加入WIFI掉线检测，联网后等待10s后，在执行MQTT重连，修复重连不在线的BUG✔
    3、把主loop()中的3个client.loop，移动到线程2中
    4、删除不必要的延时
    5、优化Serial输出代码
    6、在loop()中增加delay(10)，能降低功耗
    7、机械开关中加入推送语句，帮助平台更新正确的设备状态

    //
    //
  版本号：1.1.5
    1、机械开关 和 在线检测使用多线程，配网后，不管是否联网，都可控制开关，网络无连接时，可以使用机械开关重置网络✔
    2、尽量删除delay()函数，防止阻塞多线程运行，改用线程阻塞函数✔
    3、优化代码✔
    //
    //
  版本号：1.1.4
    1、优化代码✔
    2、添加多账号控制✔
    3、修复启动时，硬件开关闭合导致的死机的BUG✔
    4、loop()中加入delay(100)函数，降低功耗✔
    5、tosser.startup()中加入联网超时重置WIFI，防止改WIFI密码时，无法联网，不能初始化✔
    6、修复网络重连后，设备直接开启的BUG✔
    //
    //
  版本号：1.1.3
    1、等待配网时，指示灯快闪；配网成功时，设备通电1秒后关闭
    2、等待联网时，指示灯慢闪
    3、正常工作，指示灯关反馈按键开关状态
    //
    //
  版本号：1.1.2
    1、修复断电记忆模式切换无效的BUG
    //
    //
  版本号：1.1.0
    //
    //
***************************************************************************************************/
#include <ESP8266WiFi.h>      // 默认，加载WIFI头文件
#include <EEPROM.h>           // 读写EEPROM库
#include <protothreads.h>     // 多线程库文件
#include "PubSubClient.h"     // 加载MQTT库文件
#include "tosser.h"

/***************************************************************************/
//********************需要修改的部分*******************//
#define toss66_ID  "用户私钥，控制台获取"     //用户私钥，控制台获取
#define toss276_ID  "用户私钥，控制台获取"     //用户私钥，控制台获取
#define toss1210_ID  "用户私钥，控制台获取"     //用户私钥，控制台获取
/***************************************************************************/
// 主题名字，可在巴法云控制台自行创建，名称随意
const char*  topic = "yangtai002";
/***************************************************************************/
const char* mqtt_server = "bemfa.com"; //默认，MQTT服务器
const int mqtt_port = 9501;      //默认，MQTT服务器
/***************************************************************************/
// 实例化
WiFiClient espClient, espClient2, espClient3;
PubSubClient client(espClient), client2(espClient2), client3(espClient3);
/***************************************************************************/
static struct pt pt1, pt2;
/***************************************************************************/
// 初始化函数
void BtnSwitch(String msg);
//void KeySwitch();
/**************************************************************************
                        开关操作函数
***************************************************************************/
void BtnSwitch(String msg) {
  //EEPROM.begin(EEP_bgn);
  if (msg == "on") {
    Serial.println("turnOn Switch");
    digitalWrite(pinRelay, LOW);
    tosser.Count_Reset(5, 3);
    EEPROM.write(513, 0);
  } else if (msg == "off") {
    Serial.println("turnOff Switch");
    digitalWrite(pinRelay, HIGH);
    EEPROM.write(513, 1);
  } else if (msg == "keep") {
    Serial.println("Switch mode keep");
    EEPROM.write(512, byte(1));
  } else if (msg == "inon") {
    Serial.println("Switch mode on");
    EEPROM.write(512, byte(2));
  } else if (msg == "inoff") {
    Serial.println("Switch mode off");
    EEPROM.write(512, byte(3));
  }
  EEPROM.commit();
  Serial.print("EEPROM.read(512): ");
  Serial.println(EEPROM.read(512));
  Serial.print("EEPROM.read(513): ");
  Serial.println(EEPROM.read(513));
}

/**************************************************************************
       GPIO2机械开关动作判断
***************************************************************************/
/*
  void KeySwitch() {
  static bool is_btn = digitalRead(LED_BUILTIN);//按钮的标志位，用来逻辑处理对比，判断按钮有没有改变状态
  bool btn = digitalRead(LED_BUILTIN);
  if ( btn != is_btn )
  {
    Serial.println("KeySwitch change");
    // 反转开关电平
    if (digitalRead(pinRelay) == LOW) BtnSwitch("off");
    else BtnSwitch("on");
    is_btn = digitalRead(LED_BUILTIN);  //更新按钮状态
  }
  }
*/

/**************************************************************************
       mqtt消息处理，回调函数
***************************************************************************/
void callback(char* topic, byte* payload, unsigned int length) {
  Serial.print("Topic: ");
  Serial.println(topic);
  String msg = "";
  for (int i = 0; i < length; i++) {
    msg += (char)payload[i];
  }
  Serial.print("Msg: ");
  Serial.println(msg);
  BtnSwitch(msg);
  msg = "";
}


// 初始化
void setup() {
  Serial.begin(115200);
  /***************************************************************************/
  // tosser项目初始化
  tosser.Begin();
  /***************************************************************************/
  client.setServer(mqtt_server, mqtt_port);//设置mqtt服务器
  client.setCallback(callback); //mqtt消息处理
  client2.setServer(mqtt_server, mqtt_port);//设置mqtt服务器
  client2.setCallback(callback); //mqtt消息处理
  client3.setServer(mqtt_server, mqtt_port);//设置mqtt服务器
  client3.setCallback(callback); //mqtt消息处理
  Serial.println("");
  Serial.println("mqtt server is setted !");
  Serial.println("");
  Serial.print("topic: ");
  Serial.println(topic);
  /***************************************************************************/
  PT_INIT(&pt1);
  PT_INIT(&pt2);  // 3个线程初始化，其实就是pt->lc归零
}

//循环
void loop() {
  PT_SCHEDULE(Thread1(&pt1));  // 线程开始执行，不结束时，不会再次启动
  PT_SCHEDULE(Thread2(&pt2));  // 线程开始执行，不结束时，不会再次启动
  delay(10);
}

// 线程1：机械开关动作判断
int Thread1(struct pt* pt) {
  PT_BEGIN(pt);
  // 加入死循环结构，里边的代码一直循环执行，永不停息！
  while (true) {
    if ( !(bool)digitalRead(LED_BUILTIN) )
    {
      Serial.println("KeySwitch change");
      // 反转开关电平
      if (digitalRead(pinRelay) == LOW) {
        digitalWrite(pinRelay, HIGH);
        client.publish(topic, "off");
      }
      else {
        digitalWrite(pinRelay, LOW);
        client.publish(topic, "on");
      }
      while ( !(bool)digitalRead(LED_BUILTIN) ) {
        PT_SLEEP(pt, 100);
      }
    }
    PT_SLEEP(pt, 10);
  }
  // 死循环一直运行时，后面的语句不执行
  Serial.println("I`m out of while (true)");
  PT_END(pt);
}

// 线程2：MQTT检测
int Thread2(struct pt* pt) {
  PT_BEGIN(pt);
  // 加入死循环结构，里边的代码一直循环执行，永不停息！
  while (true) {
    /***************************************************************************/
    // MQTT在线检测
    while (!client.connected()) {
      if (WiFi.status() != WL_CONNECTED) {
        Serial.println("");
        Serial.println("WiFi is connectting");
        while (WiFi.status() != WL_CONNECTED) {
          Serial.print(".");
          PT_SLEEP(pt, 500);
          // 打印WiFI信息
          tosser.PrintWiFiInfo();
        }
        PT_SLEEP(pt, 10000);
      }
      Serial.println("");
      Serial.println("client: Attempting MQTT connection...");
      if (client.connect(toss66_ID)) {
        Serial.println("client.subscribe(topic)");
        //订阅主题，如果需要订阅多个主题，可发送多条订阅指令client.subscribe(topic2);client.subscribe(topic3);
        client.subscribe(topic);
      } else {
        Serial.println("client1 try again after 2 second");
        PT_SLEEP(pt, 2000);
      }
    }
    /***************************************************************************/
    while (!client2.connected()) {
      if (WiFi.status() != WL_CONNECTED) {
        Serial.println("");
        Serial.println("WiFi is connectting");
        while (WiFi.status() != WL_CONNECTED) {
          Serial.print(".");
          PT_SLEEP(pt, 500);
          // 打印WiFI信息
          tosser.PrintWiFiInfo();
        }
        PT_SLEEP(pt, 10000);
      }
      Serial.println("");
      Serial.println("client2: Attempting MQTT connection...");
      if (client2.connect(toss276_ID)) {
        Serial.println("client2.subscribe(topic)");
        client2.subscribe(topic);
      } else {
        Serial.println("client2 try again after 2 second");
        PT_SLEEP(pt, 2000);
      }
    }
    /***************************************************************************/
    while (!client3.connected()) {
      if (WiFi.status() != WL_CONNECTED) {
        Serial.println("");
        Serial.println("WiFi is connectting");
        while (WiFi.status() != WL_CONNECTED) {
          Serial.print(".");
          PT_SLEEP(pt, 500);
          // 打印WiFI信息
          tosser.PrintWiFiInfo();
        }
        PT_SLEEP(pt, 10000);
      }
      Serial.println("");
      Serial.println("client3: Attempting MQTT connection...");
      if (client3.connect(toss1210_ID)) {
        Serial.println("client3.subscribe(topic)");
        client3.subscribe(topic);
      } else {
        Serial.println("client3 try again after 2 second");
        PT_SLEEP(pt, 2000);
      }
    }
    /***************************************************************************/
    // MQTT消息检测
    client.loop();
    client2.loop();
    client3.loop();
    PT_SLEEP(pt, 10);
    /***************************************************************************/
  }
  PT_END(pt);
}
