/****************************************************************************
  智能配网
    1、等待配网时，指示灯为快闪（5fps），等待联网时，指示灯为慢闪（1fps）
    2、配网成功后，打开设备灯3秒，然后关闭，重启
    3、WIFI重置成功后，指示灯长短闪（350/150f）6次，然后重启
****************************************************************************/

#include <Arduino.h>
#include "tosser.h"
/*
  tosserClass::tosserClass() { }
  tosserClass::~tosserClass() { }
*/

/**************************************************************************
   读取EEPROM，加载WIFI相关参数（stassid、stapswd、magic）
   配网：
***************************************************************************/
void tosserClass::Begin()
{
  Serial.println("");
  Serial.println("The device is powered on!");
  /***************************************************************************/
  ReadConfig();     // 读取配置信息
  GPIO_Init();      //初始化GPIO引脚
  Serial.println("");
  Serial.println("set WiFi mode WIFI_STA.");
  WiFi.mode(WIFI_STA);    // WiFi设置为STA模式
  /***************************************************************************/
  // 如果没有配置网络，则配置->保存->重启
  if (config.magic != 0xAA) {
    pinMode(LED_BUILTIN, OUTPUT);
    SmartConfig();
  }
  // 否则接入网络
  else
  {
    pinMode(LED_BUILTIN, INPUT_PULLUP);
    WiFi.begin(config.stassid, config.stapswd);
  }
}

// 读取配置信息
void tosserClass::ReadConfig() {
  /***************************************************************************/
  Serial.println("");
  Serial.println("read EEPROM.");
  /***************************************************************************/
  // EEPROM载入内存
  EEPROM.begin(515);
  /***************************************************************************/
  // 1.1、读取WIFI相关参数
  for (int i = 0; i < sizeof(config); i++)
  {
    *(p + i) = EEPROM.read(i);
  }
  Serial.print("config.stassid: ");
  Serial.println(config.stassid);
  Serial.print("config.stapswd: ");
  Serial.println(config.stapswd);
  Serial.print("config.magic: ");
  Serial.println(config.magic, HEX);
  /***************************************************************************/
  // 1.2、读取继电器断电器状态，
  Relay_mode = byte(EEPROM.read(512));
  Relay_state = bool(EEPROM.read(513));
  Serial.print("Relay_mode: ");
  Serial.println(Relay_mode);
  Serial.print("Relay_state: ");
  Serial.println(Relay_state);
  /***************************************************************************/
  //EEPROM.end();
  /***************************************************************************/
}


// 初始化GPIO
void tosserClass::GPIO_Init() {
  /***************************************************************************/
  Serial.println("");
  Serial.println("GPIO Init.");
  /***************************************************************************/
  // 初始化pinRelay引脚模式
  Serial.println("set pinRelay mode OUTPUT.");
  pinMode(pinRelay, OUTPUT);
  if (Relay_mode == byte(1) && !Relay_state || Relay_mode == byte(2)) {
    digitalWrite(pinRelay, LOW);
  } else {
    digitalWrite(pinRelay, HIGH);
  }
  /***************************************************************************/
}


// 配置联网信息,并写入EEPROM
void tosserClass::SmartConfig() {
  /***************************************************************************/
  Serial.println("");
  Serial.println("waiting SmartConfig.");
  /***************************************************************************/
  WiFi.beginSmartConfig();
  /***************************************************************************/
  int cnt = 0;    // 循环体运行次数
  while (true) {
    /***************************************************************************/
    ESP.wdtFeed();
    /***************************************************************************/
    cnt++;
    // 等待配置网络......（指示灯为快闪状态）
    for (int i = 0; i < 10; i++) {
      digitalWrite(LED_BUILTIN, !digitalRead(LED_BUILTIN));
      delay(100);
    }
    Serial.print(".");
    /***************************************************************************/
    // 如果smartConfig智能配网完成，保存配置，延时重启
    if (WiFi.smartConfigDone()) {
      // 配网完成，指示灯常亮，设备状态开
      digitalWrite(LED_BUILTIN, LOW);
      digitalWrite(pinRelay, LOW);
      strcpy(config.stassid, WiFi.SSID().c_str());    // WIFI名称
      strcpy(config.stapswd, WiFi.psk().c_str());     // WIFI密码
      config.magic = 0xAA;                            // 运行模式
      // 网络配置信息写入EEPROM
      Serial.println("");
      Serial.println("SmartConfig done, Writing config to EEPROM.");
      //EEPROM.begin(515);
      for (int i = 0; i < sizeof(config); i++)
      {
        EEPROM.write(i, *(p + i));
      }
      EEPROM.end();
      // 重启
      Serial.println("Rebooting...");
      Serial.println("");
      Serial.println("");
      ESP.restart();
    }
    /***************************************************************************/
    // 如果超时没有配置成功，重启
    if (cnt >= 120) {
      Serial.println("Write config failed! Rebooting...");
      Serial.println("");
      Serial.println("");
      ESP.restart();
    }
    /***************************************************************************/
  }
}

/***************************************************************************
    开始联网
***************************************************************************/
void tosserClass::joinWiFi() {
  /***************************************************************************/
  // 连接路由器
  Serial.println("");
  Serial.println("WiFi is connectting");
  Serial.println("");
  WiFi.begin(config.stassid, config.stapswd);
  WiFi.setAutoConnect(true);
  /***************************************************************************/
}

/***************************************************************************
    打印WiFI信息
***************************************************************************/
void tosserClass::PrintWiFiInfo() {
  /***************************************************************************/
  if (WiFi.status() == WL_CONNECTED) {
    // 联网成功，打印网址
    Serial.println("");
    Serial.print("Station IP: ");
    Serial.println(WiFi.localIP());
    uint8_t mac[5];
    WiFi.macAddress(mac);
    Serial.print("MAC: ");
    for (int i = 0; i < 6; i++) {
      if (mac[i] < 16) Serial.print("0");
      Serial.print(mac[i], HEX);
      if (i != 5) Serial.print("-");
      else {
        Serial.println("");
        Serial.println("wifi connected ok !");
      }
    }
  }
  /***************************************************************************/
}



/***********************************************************
    重置WIFI配置，然后延时重启系统
***********************************************************/
void tosserClass::WiFiReset() {
  /***************************************************************************/
  Serial.println("\r\n Restore Factory....... ");
  strcpy(config.stassid, "");
  strcpy(config.stapswd, "");
  config.magic = 0x00;
  /***************************************************************************/
  // 网络配置信息写入EEPROM
  EEPROM.begin(515);
  for (int i = 0; i < sizeof(config); i++)
  {
    EEPROM.write(i, *(p + i));
  }
  EEPROM.end();
  /***************************************************************************/
  ESP.wdtFeed();
  // 重启
  ESP.restart();
  /***************************************************************************/
}


/**************************************************************************
       Time_interval时间内开关Inr_cnt次
***************************************************************************/
void tosserClass::Count_Reset(int Inr_sec, int Inr_cnt) {
  count++;
  Serial.print("count:");
  Serial.println(count);
  if (count == 1) {
    // 如果count值为1，设置开始时间
    Serial.println("Time_start reset");
    Time_start = millis();
  }
  uint32_t nowTime = millis();
  // 时间不能溢出
  if (nowTime > Time_start) {
    // 超时重置count
    if (nowTime - Time_start > Inr_sec * 1000) {
      count = 0;
    }
    // 如果count值为Inr_cnt，则重置WIFI
    else if (count == Inr_cnt) {
      Serial.println("tosser wifiReset");
      WiFiReset(); // 重置WIFI函数
    }
  }
}


#if !defined(NO_GLOBAL_INSTANCES) && !defined(NO_GLOBAL_tosser)
tosserClass tosser;
#endif
